// -*- Mode: c++ -*-

/*
 * Copyright (C) 2001 NODA, Itsuki, CARC, AIST, JAPAN
 * Copyright (C) 1999, 2000 Itsuki Noda, Electrotechnical Laboratory, Japan
 */

#ifndef _itk_WithDescriber_h_
#define _itk_WithDescriber_h_
//////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include <iomanip.h>

#include "itk/btype.h"

namespace Itk {

   class WithDescriber {
      public:
	 virtual const char* className() const { return "WithDescriber" ; } ;

      public:
	 virtual ~WithDescriber() {} ;

      public:
	 virtual void describe(ostream& ostr, 
			       const Bool detailp = True) const {
	    ostr << "#" << className() << "[" << Ptr(this) << "]" ;
	    if(detailp) ostr << endl ;
	 } ;

      public:
	 virtual void describePtr(ostream& ostr, 
				  const Bool detailp = True) const {
	    ostr << Ptr(this) ;
	 } ;
   } ;

   inline ostream& operator<<(ostream& ostr, const WithDescriber& dt) {
      dt.describe(ostr,False) ;
      return ostr ;
   } 

   inline ostream& operator<<(ostream& ostr, const WithDescriber* ptr) {
      if(isNull(ptr)) ostr << Ptr(ptr) ;
      else ptr->describePtr(ostr,False) ;
      return ostr ;
   } 

   inline void dbg(const WithDescriber & obj) {
      obj.describe(cerr) ;
   } ;

   inline void dbg(const WithDescriber * obj) {
      obj->describe(cerr) ;
   } ;
}

//////////////////////////////////////////////////////////////////////
#endif
